import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _typeof from "@babel/runtime/helpers/typeof";
import React from 'react';
import { isCollection, isIndexed, isKeyed } from 'immutable';
import isIterable from '../utils/isIterable';
var IS_IMMUTABLE_KEY = '@@__IS_IMMUTABLE__@@';

function isImmutable(value) {
  return isKeyed(value) || isIndexed(value) || isCollection(value);
}

function getShortTypeString(val, diff) {
  if (diff && Array.isArray(val)) {
    val = val[val.length === 2 ? 1 : 0];
  }

  if (isIterable(val) && !isImmutable(val)) {
    return '(…)';
  } else if (Array.isArray(val)) {
    return val.length > 0 ? '[…]' : '[]';
  } else if (val === null) {
    return 'null';
  } else if (val === undefined) {
    return 'undef';
  } else if (_typeof(val) === 'object') {
    return Object.keys(val).length > 0 ? '{…}' : '{}';
  } else if (typeof val === 'function') {
    return 'fn';
  } else if (typeof val === 'string') {
    return "\"".concat(val.substr(0, 10) + (val.length > 10 ? '…' : ''), "\"");
  } else if (_typeof(val) === 'symbol') {
    return 'symbol';
  } else {
    return val;
  }
}

function getText(type, data, previewContent, isDiff) {
  if (type === 'Object') {
    // eslint-disable-next-line @typescript-eslint/ban-types
    var keys = Object.keys(data);
    if (!previewContent) return keys.length ? '{…}' : '{}';
    var str = keys.slice(0, 3).map(function (key) {
      return "".concat(key, ": ").concat(getShortTypeString(data[key], isDiff));
    }).concat(keys.length > 3 ? ['…'] : []).join(', ');
    return "{ ".concat(str, " }");
  } else if (type === 'Array') {
    if (!previewContent) return data.length ? '[…]' : '[]';

    var _str = data.slice(0, 4).map(function (val) {
      return getShortTypeString(val, isDiff);
    }).concat(data.length > 4 ? ['…'] : []).join(', ');

    return "[".concat(_str, "]");
  } else {
    return type;
  }
}

export function getItemString(styling, type, data, dataTypeKey, previewContent, isDiff) {
  return /*#__PURE__*/React.createElement("span", styling('treeItemHint'), data[IS_IMMUTABLE_KEY] ? 'Immutable' : '', dataTypeKey && data[dataTypeKey] ? "".concat(data[dataTypeKey], " ") : '', getText(type, data, previewContent, isDiff));
}
export function createTreeItemLabelRenderer(styling) {
  return function labelRenderer(_ref, nodeType, expanded) {
    var _ref2 = _slicedToArray(_ref, 1),
        key = _ref2[0];

    return /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement("span", styling('treeItemKey'), key), !expanded && ': ');
  };
}